@@ -1,6 +1,7 @@
 import errno
 import logging
 import os
+import gc
 from circus.fixed_threading import Thread, get_ident
 import sys
 from time import sleep
@@ -67,9 +68,13 @@ class Arbiter(object):
       to /dev/null. (default: False)
     - **debug** -- if True, adds a lot of debug info in the stdout (default:
       False)
+    - **debug_gc** -- if True, does gc.set_debug(gc.DEBUG_LEAK) (default:
+      False)
+      to circusd to analyze problems (default: False)
     - **proc_name** -- the arbiter process name
     - **fqdn_prefix** -- a prefix for the unique identifier of the circus
                          instance on the cluster.
+    - **endpoint_owner** -- unix user to chown the endpoint to if using ipc.
     """
 
     def __init__(self, watchers, endpoint, pubsub_endpoint, check_delay=1.0,
@@ -78,9 +83,10 @@ class Arbiter(object):
                  multicast_endpoint=None, plugins=None,
                  sockets=None, warmup_delay=0, httpd=False,
                  httpd_host='localhost', httpd_port=8080,
-                 httpd_close_outputs=False, debug=False,
+                 httpd_close_outputs=False, debug=False, debug_gc=False,
                  ssh_server=None, proc_name='circusd', pidfile=None,
-                 loglevel=None, logoutput=None, fqdn_prefix=None):
+                 loglevel=None, logoutput=None, loggerconfig=None,
+                 fqdn_prefix=None, umask=None, endpoint_owner=None):
 
         self.watchers = watchers
         self.endpoint = endpoint
@@ -94,7 +100,10 @@ class Arbiter(object):
         self.pidfile = pidfile
         self.loglevel = loglevel
         self.logoutput = logoutput
-
+        self.loggerconfig = loggerconfig
+        self.umask = umask
+        self.endpoint_owner = endpoint_owner
+        self._running = False
         try:
             # getfqdn appears to fail in Python3.3 in the unittest
             # framework so fall back to gethostname
@@ -127,6 +136,10 @@ class Arbiter(object):
         else:
             self.stdout_stream = self.stderr_stream = None
 
+        self.debug_gc = debug_gc
+        if debug_gc:
+            gc.set_debug(gc.DEBUG_LEAK)
+
         # initializing circusd-stats as a watcher when configured
         self.statsd = statsd
         self.stats_endpoint = stats_endpoint
@@ -141,6 +154,10 @@ class Arbiter(object):
                 cmd += ' --ssh %s' % ssh_server
             if debug:
                 cmd += ' --log-level DEBUG'
+            elif self.loglevel:
+                cmd += ' --log-level ' + self.loglevel
+            if self.logoutput:
+                cmd += ' --log-output ' + self.logoutput
             stats_watcher = Watcher('circusd-stats', cmd, use_sockets=True,
                                     singleton=True,
                                     stdout_stream=self.stdout_stream,
@@ -187,7 +204,9 @@ class Arbiter(object):
                 fqn = plugin['use']
                 cmd = get_plugin_cmd(plugin, self.endpoint,
                                      self.pubsub_endpoint, self.check_delay,
-                                     ssh_server, debug=self.debug)
+                                     ssh_server, debug=self.debug,
+                                     loglevel=self.loglevel,
+                                     logoutput=self.logoutput)
                 plugin_cfg = dict(cmd=cmd, priority=1, singleton=True,
                                   stdout_stream=self.stdout_stream,
                                   stderr_stream=self.stderr_stream,
@@ -204,13 +223,18 @@ class Arbiter(object):
         self.sockets = CircusSockets(sockets)
         self.warmup_delay = warmup_delay
 
+    @property
+    def running(self):
+        return self._running
+
     def _init_context(self, context):
         self.context = context or zmq.Context.instance()
         if self.loop is None:
             ioloop.install()
             self.loop = ioloop.IOLoop.instance()
         self.ctrl = Controller(self.endpoint, self.multicast_endpoint,
-                               self.context, self.loop, self, self.check_delay)
+                               self.context, self.loop, self, self.check_delay,
+                               self.endpoint_owner)
 
     def get_socket(self, name):
         return self.sockets.get(name, None)
@@ -390,8 +414,8 @@ class Arbiter(object):
             watchers.append(Watcher.load_from_config(watcher))
 
         sockets = []
-        for socket in cfg.get('sockets', []):
-            sockets.append(CircusSocket.load_from_config(socket))
+        for socket_ in cfg.get('sockets', []):
+            sockets.append(CircusSocket.load_from_config(socket_))
 
         httpd = cfg.get('httpd', False)
         if httpd:
@@ -417,11 +441,15 @@ class Arbiter(object):
                       httpd_host=cfg.get('httpd_host', 'localhost'),
                       httpd_port=cfg.get('httpd_port', 8080),
                       debug=cfg.get('debug', False),
+                      debug_gc=cfg.get('debug_gc', False),
                       ssh_server=cfg.get('ssh_server', None),
                       pidfile=cfg.get('pidfile', None),
                       loglevel=cfg.get('loglevel', None),
                       logoutput=cfg.get('logoutput', None),
-                      fqdn_prefix=cfg.get('fqdn_prefix', None))
+                      loggerconfig=cfg.get('loggerconfig', None),
+                      fqdn_prefix=cfg.get('fqdn_prefix', None),
+                      umask=cfg['umask'],
+                      endpoint_owner=cfg.get('endpoint_owner', None))
 
         # store the cfg which will be used, so it can be used later
         # for checking if the cfg has been changed
@@ -438,6 +466,10 @@ class Arbiter(object):
         # set process title
         _setproctitle(self.proc_name)
 
+        # set umask even though we may have already set it early in circusd.py
+        if self.umask is not None:
+            os.umask(self.umask)
+
         # event pub socket
         self.evpub_socket = self.context.socket(zmq.PUB)
         self.evpub_socket.bind(self.pubsub_endpoint)
@@ -489,6 +521,7 @@ class Arbiter(object):
                 # start_watchers will be called just after the start_io_loop()
                 self.loop.add_future(self.start_watchers(), lambda x: None)
             logger.info('Arbiter now waiting for commands')
+            self._running = True
             if not self._provided_loop:
                 # If an event loop is not provided, block at this line
                 self.start_io_loop()
@@ -501,9 +534,12 @@ class Arbiter(object):
     def stop_controller_and_close_sockets(self):
         self.ctrl.stop()
         self.evpub_socket.close()
+
         if len(self.sockets) > 0:
             self.sockets.close_all()
 
+        self._running = False
+
     def start_io_loop(self):
         """Starts the ioloop and wait inside it
         """
@@ -536,7 +572,7 @@ class Arbiter(object):
     def _stop(self):
         logger.info('Arbiter exiting')
         self._stopping = True
-        yield self._stop_watchers()
+        yield self._stop_watchers(close_output_streams=True)
         if self._provided_loop:
             cb = self.stop_controller_and_close_sockets
             self.loop.add_callback(cb)
@@ -586,10 +622,13 @@ class Arbiter(object):
         need_on_demand = False
         # manage and reap processes
         self.reap_processes()
+        list_to_yield = []
         for watcher in self.iter_watchers():
             if watcher.on_demand and watcher.is_stopped():
                 need_on_demand = True
-            yield watcher.manage_processes()
+            list_to_yield.append(watcher.manage_processes())
+        if len(list_to_yield) > 0:
+            yield list_to_yield
 
         if need_on_demand:
             sockets = [x.fileno() for x in self.sockets.values()]
@@ -602,7 +641,7 @@ class Arbiter(object):
     @synchronized("arbiter_reload")
     @gen.coroutine
     @debuglog
-    def reload(self, graceful=True):
+    def reload(self, graceful=True, sequential=False):
         """Reloads everything.
 
         Run the :func:`prereload_fn` callable if any, then gracefuly
@@ -623,7 +662,7 @@ class Arbiter(object):
 
         # gracefully reload watchers
         for watcher in self.iter_watchers():
-            yield watcher._reload(graceful=graceful)
+            yield watcher._reload(graceful=graceful, sequential=sequential)
             tornado_sleep(self.warmup_delay)
 
     def numprocesses(self):
@@ -691,13 +730,15 @@ class Arbiter(object):
     @gen.coroutine
     def _start_watchers(self):
         for watcher in self.iter_watchers():
-            yield watcher._start()
-            yield tornado_sleep(self.warmup_delay)
+            if watcher.autostart:
+                yield watcher._start()
+                yield tornado_sleep(self.warmup_delay)
 
     @gen.coroutine
     @debuglog
-    def _stop_watchers(self):
-        yield [w._stop() for w in self.iter_watchers(reverse=False)]
+    def _stop_watchers(self, close_output_streams=False):
+        yield [w._stop(close_output_streams)
+               for w in self.iter_watchers(reverse=False)]
 
     @synchronized("arbiter_stop_watchers")
     @gen.coroutine
@@ -718,6 +759,10 @@ class Arbiter(object):
     def restart(self, inside_circusd=False):
         yield self._restart(inside_circusd=inside_circusd)
 
+    @property
+    def endpoint_owner_mode(self):
+        return self.ctrl.endpoint_owner_mode  # just wrap the controller
+
 
 class ThreadedArbiter(Thread, Arbiter):
 
