@@ -117,7 +117,7 @@ class Signal(Command):
             'signum': args[-1],
         }
         if len(args) == 3:
-            props['pid'] = args[1]
+            props['pid'] = int(args[1])
         return self.make_message(**props)
 
     def execute(self, arbiter, props):
@@ -145,7 +145,7 @@ class Signal(Command):
     def validate(self, props):
         super(Signal, self).validate(props)
 
-        if 'childpid' in props and not 'pid' in props:
+        if 'childpid' in props and 'pid' not in props:
             raise ArgumentError('cannot specify childpid without pid')
 
         try:
