@@ -1,11 +1,13 @@
 from circus.exc import ArgumentError, MessageError
 from circus.py3compat import string_types
 from circus import util
+import resource
 import warnings
 
 
 _HOOKS = ('before_start', 'after_start', 'before_stop', 'after_stop',
-          'before_spawn', 'before_signal', 'after_signal')
+          'before_spawn', 'after_spawn', 'before_signal', 'after_signal',
+          'extended_stats')
 
 
 def convert_option(key, val):
@@ -47,6 +49,8 @@ def convert_option(key, val):
         return int(val)
     elif key == 'respawn':
         return util.to_bool(val)
+    elif key == "singleton":
+        return util.to_bool(val)
     elif key.startswith('stderr_stream.') or key.startswith('stdout_stream.'):
         subkey = key.split('.', 1)[-1]
         if subkey in ('max_bytes', 'backup_count'):
@@ -74,6 +78,8 @@ def convert_option(key, val):
         if name not in _HOOKS:
             raise ArgumentError(name)
         return val
+    elif key.startswith('rlimit_'):
+        return int(val)
 
     raise ArgumentError("unknown key %r" % key)
 
@@ -84,13 +90,13 @@ def validate_option(key, val):
                   'shell', 'env', 'cmd', 'args', 'copy_env', 'retry_in',
                   'max_retry', 'graceful_timeout', 'stdout_stream',
                   'stderr_stream', 'max_age', 'max_age_variance', 'respawn',
-                  'hooks')
+                  'singleton', 'hooks')
 
-    valid_prefixes = ('stdout_stream', 'stderr_stream')
+    valid_prefixes = ('stdout_stream.', 'stderr_stream.', 'hooks.', 'rlimit_')
 
     def _valid_prefix():
         for prefix in valid_prefixes:
-            if key.startswith('%s.' % prefix):
+            if key.startswith('%s' % prefix):
                 return True
         return False
 
@@ -102,19 +108,19 @@ def validate_option(key, val):
         if not isinstance(val, int):
             raise MessageError("%r isn't an integer" % key)
 
-    if key in ('warmup_delay', 'retry_in', 'graceful_timeout',):
+    elif key in ('warmup_delay', 'retry_in', 'graceful_timeout',):
         if not isinstance(val, (int, float)):
             raise MessageError("%r isn't a number" % key)
 
-    if key in ('uid', 'gid',):
+    elif key in ('uid', 'gid',):
         if not isinstance(val, int) and not isinstance(val, string_types):
             raise MessageError("%r isn't an integer or string" % key)
 
-    if key in ('send_hup', 'shell', 'copy_env', 'respawn', 'stop_children'):
+    elif key in ('send_hup', 'shell', 'copy_env', 'respawn', 'stop_children'):
         if not isinstance(val, bool):
             raise MessageError("%r isn't a valid boolean" % key)
 
-    if key in ('env', ):
+    elif key in ('env', ):
         if not isinstance(val, dict):
             raise MessageError("%r isn't a valid object" % key)
 
@@ -122,7 +128,7 @@ def validate_option(key, val):
             if not isinstance(v, string_types):
                 raise MessageError("%r isn't a string" % k)
 
-    if key == 'hooks':
+    elif key == 'hooks':
         if not isinstance(val, dict):
             raise MessageError("%r isn't a valid hook dict" % val)
 
@@ -130,11 +136,19 @@ def validate_option(key, val):
             if key not in _HOOKS:
                 raise MessageError("Unknown hook %r" % val)
 
-    if key in ('stderr_stream', 'stdout_stream'):
+    elif key in ('stderr_stream', 'stdout_stream'):
         if not isinstance(val, dict):
             raise MessageError("%r isn't a valid object" % key)
-        if not 'class' in val:
+        if 'class' not in val:
             raise MessageError("%r must have a 'class' key" % key)
         if 'refresh_time' in val:
             warnings.warn("'refresh_time' is deprecated and not useful "
                           "anymore for %r" % key)
+
+    elif key.startswith('rlimit_'):
+        rlimit_key = key[7:]
+        rlimit_int = getattr(resource, 'RLIMIT_' + rlimit_key.upper(), None)
+        if rlimit_int is None:
+            raise MessageError("%r isn't a valid rlimit setting" % key)
+        if not isinstance(val, int):
+            raise MessageError("%r rlimit value isn't a valid int" % val)
