@@ -29,7 +29,7 @@ class Pidfile(object):
         if fdir and not os.path.isdir(fdir):
             raise RuntimeError("%s doesn't exist. Can't create pidfile" % fdir)
         fd, fname = tempfile.mkstemp(dir=fdir)
-        os.write(fd, "%s\n" % self.pid)
+        os.write(fd, "{0}\n".format(self.pid).encode('utf-8'))
         if self.fname:
             os.rename(fname, self.fname)
         else:
@@ -70,10 +70,10 @@ class Pidfile(object):
                     os.kill(wpid, 0)
                     return wpid
                 except OSError as e:
-                    if e[0] == errno.ESRCH:
+                    if e.args[0] == errno.ESRCH:
                         return
                     raise
         except IOError as e:
-            if e[0] == errno.ENOENT:
+            if e.args[0] == errno.ENOENT:
                 return
             raise
