@@ -19,10 +19,11 @@ import warnings
 
 from psutil import Popen, STATUS_ZOMBIE, STATUS_DEAD, NoSuchProcess
 
-from circus.py3compat import bytestring, string_types
+from circus.py3compat import bytestring, string_types, quote
 from circus.sockets import CircusSocket
 from circus.util import (get_info, to_uid, to_gid, debuglog, get_working_dir,
-                         ObjectDict, replace_gnu_args)
+                         ObjectDict, replace_gnu_args, is_win, get_default_gid,
+                         get_username_from_uid)
 from circus import logger
 
 
@@ -36,6 +37,77 @@ UNEXISTING = 2
 OTHER = 3
 
 
+# psutil < 2.x compat
+def get_children(proc, recursive=False):
+    try:
+        return proc.children(recursive)
+    except AttributeError:
+        return proc.get_children(recursive)
+
+
+def get_memory_info(proc):
+    try:
+        return proc.memory_info()
+    except AttributeError:
+        return proc.get_memory_info()
+
+
+def get_cpu_percent(proc, **kw):
+    try:
+        return proc.cpu_percent(**kw)
+    except AttributeError:
+        return proc.get_cpu_percent(**kw)
+
+
+def get_memory_percent(proc):
+    try:
+        return proc.memory_percent()
+    except AttributeError:
+        return proc.get_memory_percent()
+
+
+def get_cpu_times(proc):
+    try:
+        return proc.cpu_times()
+    except AttributeError:
+        return proc.get_cpu_times()
+
+
+def get_nice(proc):
+    try:
+        return proc.nice()
+    except (AttributeError, TypeError):
+        return proc.get_nice()
+
+
+def get_cmdline(proc):
+    try:
+        return proc.cmdline()
+    except TypeError:
+        return proc.cmdline
+
+
+def get_create_time(proc):
+    try:
+        return proc.create_time()
+    except TypeError:
+        return proc.create_time
+
+
+def get_username(proc):
+    try:
+        return proc.username()
+    except TypeError:
+        return proc.username
+
+
+def get_status(proc):
+    try:
+        return proc.status()
+    except TypeError:
+        return proc.status
+
+
 class Process(object):
     """Wraps a process.
 
@@ -101,7 +173,12 @@ class Process(object):
         self.args = args
         self.working_dir = working_dir or get_working_dir()
         self.shell = shell
-        self.uid = to_uid(uid) if uid else None
+        if uid:
+            self.uid = to_uid(uid)
+            self.username = get_username_from_uid(self.uid)
+        else:
+            self.username = None
+            self.uid = None
         self.gid = to_gid(gid) if gid else None
         self.env = env or {}
         self.rlimits = rlimits or {}
@@ -116,6 +193,9 @@ class Process(object):
         # sockets created before fork, should be let go after.
         self._sockets = []
 
+        if self.uid is not None and self.gid is None:
+            self.gid = get_default_gid(self.uid)
+
         if spawn:
             self.spawn()
 
@@ -195,6 +275,7 @@ class Process(object):
                     # versions of python < 2.6.2 don't manage unsigned int for
                     # groups like on osx or fedora
                     os.setgid(-ctypes.c_int(-self.gid).value)
+                os.initgroups(self.username, self.gid)
 
             if self.uid:
                 os.setuid(self.uid)
@@ -261,9 +342,30 @@ class Process(object):
         else:
             args = shlex.split(bytestring(cmd))
 
+        if self.shell:
+            # subprocess.Popen(shell=True) implies that 1st arg is the
+            # requested command, remaining args are applied to sh.
+            args = [' '.join(quote(arg) for arg in args)]
+            shell_args = format_kwargs.get('shell_args', None)
+            if shell_args and is_win():
+                logger.warn("shell_args won't apply for "
+                            "windows platforms: %s", shell_args)
+            elif isinstance(shell_args, string_types):
+                args += shlex.split(bytestring(replace_gnu_args(
+                    shell_args, **format_kwargs)))
+            elif shell_args:
+                args += [bytestring(replace_gnu_args(arg, **format_kwargs))
+                         for arg in shell_args]
+
+        elif format_kwargs.get('shell_args', False):
+            logger.warn("shell_args is defined but won't be used "
+                        "in this context: %s", format_kwargs['shell_args'])
         logger.debug("process args: %s", args)
         return args
 
+    def returncode(self):
+        return self._worker.returncode
+
     @debuglog
     def poll(self):
         return self._worker.poll()
@@ -331,18 +433,18 @@ class Process(object):
         info["started"] = self.started
         info["children"] = []
         info['wid'] = self.wid
-        for child in self._worker.get_children():
+        for child in get_children(self._worker):
             info["children"].append(get_info(child))
 
         return info
 
     def children(self):
         """Return a list of children pids."""
-        return [child.pid for child in self._worker.get_children()]
+        return [child.pid for child in get_children(self._worker)]
 
     def is_child(self, pid):
         """Return True is the given *pid* is a child of that process."""
-        pids = [child.pid for child in self._worker.get_children()]
+        pids = [child.pid for child in get_children(self._worker)]
         if pid in pids:
             return True
         return False
@@ -351,7 +453,7 @@ class Process(object):
     def send_signal_child(self, pid, signum):
         """Send signal *signum* to child *pid*."""
         children = dict((child.pid, child)
-                        for child in self._worker.get_children())
+                        for child in get_children(self._worker))
         try:
             children[pid].send_signal(signum)
         except KeyError:
@@ -360,7 +462,7 @@ class Process(object):
     @debuglog
     def send_signal_children(self, signum):
         """Send signal *signum* to all children."""
-        for child in self._worker.get_children():
+        for child in get_children(self._worker):
             try:
                 child.send_signal(signum)
             except OSError as e:
@@ -377,7 +479,7 @@ class Process(object):
         - OTHER
         """
         try:
-            if self._worker.status in (STATUS_ZOMBIE, STATUS_DEAD):
+            if get_status(self._worker) in (STATUS_ZOMBIE, STATUS_DEAD):
                 return DEAD_OR_ZOMBIE
         except NoSuchProcess:
             return UNEXISTING
