@@ -14,13 +14,14 @@ from circus.arbiter import Arbiter
 from circus.client import CircusClient
 from circus.plugins import CircusPlugin
 from circus.tests.support import TestCircus, async_poll_for, truncate_file
-from circus.tests.support import EasyTestSuite, skipIf
+from circus.tests.support import EasyTestSuite, skipIf, get_ioloop
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_MULTICAST,
                          DEFAULT_ENDPOINT_SUB)
 from circus.watcher import Watcher
-from circus.tests.support import has_circusweb, poll_for_callable
+from circus.tests.support import (has_circusweb, poll_for_callable,
+                                  get_available_port)
 from circus import watcher as watcher_mod
-from zmq.utils.strtypes import u
+from circus.py3compat import s
 
 
 _GENERIC = os.path.join(os.path.dirname(__file__), 'generic.py')
@@ -36,7 +37,7 @@ class Plugin(CircusPlugin):
 
     def handle_recv(self, data):
         topic, msg = data
-        topic_parts = u(topic).split(".")
+        topic_parts = s(topic).split(".")
         watcher = topic_parts[1]
         action = topic_parts[2]
         with open(self.config['file'], 'a+') as f:
@@ -290,6 +291,23 @@ class TestTrainer(TestCircus):
         self.assertNotEqual(processes1, processes2)
         yield self.stop_arbiter()
 
+    @tornado.testing.gen_test
+    def test_reload_sequential(self):
+        yield self.start_arbiter(graceful_timeout=0)
+        name = 'test_reload_sequential'
+        options = self._get_options(numprocesses=4)
+        yield self._call("add", name=name, cmd=self._get_cmd(),
+                         start=True, options=options)
+        resp = yield self._call("list", name=name)
+        processes1 = resp.get('pids')
+        truncate_file(self.test_file)  # clean slate
+        yield self._call("reload", sequential=True)
+        self.assertTrue(async_poll_for(self.test_file, 'START'))  # restarted
+        resp = yield self._call("list", name=name)
+        processes2 = resp.get('pids')
+        self.assertNotEqual(processes1, processes2)
+        yield self.stop_arbiter()
+
     @tornado.testing.gen_test
     def test_reload2(self):
         yield self.start_arbiter(graceful_timeout=0)
@@ -513,7 +531,7 @@ class TestTrainer(TestCircus):
         @tornado.gen.coroutine
         def _sleep(duration):
             called.append(duration)
-            loop = tornado.ioloop.IOLoop.instance()
+            loop = get_ioloop()
             yield tornado.gen.Task(loop.add_timeout, time() + duration)
 
         watcher_mod.tornado_sleep = _sleep
@@ -536,6 +554,9 @@ class MockWatcher(Watcher):
     def start(self):
         self.started = True
 
+    def spawn_process(self):
+        self.processes[1] = 'dummy'
+
 
 class TestArbiter(TestCircus):
     """
@@ -558,8 +579,9 @@ class TestArbiter(TestCircus):
 
     @tornado.testing.gen_test
     def test_add_watcher(self):
-        arbiter = Arbiter([], DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
-                          loop=tornado.ioloop.IOLoop.instance(),
+        controller = "tcp://127.0.0.1:%d" % get_available_port()
+        sub = "tcp://127.0.0.1:%d" % get_available_port()
+        arbiter = Arbiter([], controller, sub, loop=get_ioloop(),
                           check_delay=-1)
         arbiter.add_watcher('foo', 'sleep 5')
         try:
@@ -568,14 +590,28 @@ class TestArbiter(TestCircus):
         finally:
             yield arbiter.stop()
 
+    @tornado.testing.gen_test
+    def test_start_arbiter_with_autostart(self):
+        arbiter = Arbiter([], DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
+                          loop=get_ioloop(),
+                          check_delay=-1)
+        arbiter.add_watcher('foo', 'sleep 5', autostart=False)
+        try:
+            yield arbiter.start()
+            self.assertEqual(arbiter.watchers[0].status(), 'stopped')
+        finally:
+            yield arbiter.stop()
+
 
 @skipIf(not has_circusweb(), 'Tests for circus-web')
 class TestCircusWeb(TestCircus):
 
     @tornado.testing.gen_test
     def test_circushttpd(self):
-        arbiter = Arbiter([], DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
-                          loop=tornado.ioloop.IOLoop.instance(),
+        controller = "tcp://127.0.0.1:%d" % get_available_port()
+        sub = "tcp://127.0.0.1:%d" % get_available_port()
+
+        arbiter = Arbiter([], controller, sub, loop=get_ioloop(),
                           check_delay=-1, httpd=True, debug=True)
         self.arbiters.append(arbiter)
         try:
