@@ -22,6 +22,30 @@ def so_bindtodevice_supported():
 
 class TestSockets(TestCase):
 
+    def setUp(self):
+        super(TestSockets, self).setUp()
+        self.files = []
+
+    def tearDown(self):
+        for file_ in self.files:
+            if os.path.exists(file_):
+                os.remove(file_)
+
+        super(TestSockets, self).tearDown()
+
+    def _get_file(self):
+        fd, _file = tempfile.mkstemp()
+        os.close(fd)
+        self.files.append(_file)
+        return _file
+
+    def _get_tmp_filename(self):
+        # XXX horrible way to get a filename
+        fd, _file = tempfile.mkstemp()
+        os.close(fd)
+        os.remove(_file)
+        return _file
+
     def test_socket(self):
         sock = CircusSocket('somename', 'localhost', 0)
         try:
@@ -56,10 +80,7 @@ class TestSockets(TestCase):
         self.assertRaises(socket.error, CircusSocket.load_from_config, config)
 
     def test_load_from_config_umask(self):
-        fd, sockfile = tempfile.mkstemp()
-        os.close(fd)
-        os.remove(sockfile)
-
+        sockfile = self._get_tmp_filename()
         config = {'name': 'somename', 'path': sockfile, 'umask': 0}
         sock = CircusSocket.load_from_config(config)
         try:
@@ -67,11 +88,26 @@ class TestSockets(TestCase):
         finally:
             sock.close()
 
-    def test_unix_socket(self):
-        fd, sockfile = tempfile.mkstemp()
-        os.close(fd)
-        os.remove(sockfile)
+    def test_load_from_config_replace(self):
+        sockfile = self._get_file()
 
+        config = {'name': 'somename', 'path': sockfile, 'replace': False}
+        sock = CircusSocket.load_from_config(config)
+        try:
+            self.assertRaises(OSError, sock.bind_and_listen)
+        finally:
+            sock.close()
+
+        config = {'name': 'somename', 'path': sockfile, 'replace': True}
+        sock = CircusSocket.load_from_config(config)
+        sock.bind_and_listen()
+        try:
+            self.assertEqual(sock.replace, True)
+        finally:
+            sock.close()
+
+    def test_unix_socket(self):
+        sockfile = self._get_tmp_filename()
         sock = CircusSocket('somename', path=sockfile, umask=0)
         try:
             sock.bind_and_listen()
@@ -83,10 +119,7 @@ class TestSockets(TestCase):
 
     def test_unix_cleanup(self):
         sockets = CircusSockets()
-        fd, sockfile = tempfile.mkstemp()
-        os.close(fd)
-        os.remove(sockfile)
-
+        sockfile = self._get_tmp_filename()
         try:
             sockets.add('unix', path=sockfile)
             sockets.bind_and_listen_all()
@@ -133,7 +166,13 @@ class TestSockets(TestCase):
                   'so_reuseport': True}
 
         sock = CircusSocket.load_from_config(config)
-        sockopt = sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT)
+        try:
+            sockopt = sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT)
+        except socket.error:
+            # see #699
+            return
+        finally:
+            sock.close()
 
         self.assertEqual(sock.so_reuseport, True)
         self.assertNotEqual(sockopt, 0)
@@ -152,6 +191,7 @@ class TestSockets(TestCase):
         finally:
             if saved is not None:
                 socket.SO_REUSEPORT = saved
+            sock.close()
 
 
 test_suite = EasyTestSuite(__name__)
